# This perl module contains the configuration information for this parser.
# If you don't understand the format of this configuration file DON'T TOUCH IT.
#
# My Vendor Interface
# 
#*******************************************************************************
# Licensed Materials - Property of IBM
#
# 5724-T01 5724-S98
#
# (C) Copyright IBM Corp. 2003, 2008. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#*******************************************************************************



package EngineConfig;


@ISA = qw(Exporter);
@EXPORT = qw(engine_config);

#use diagnostics;
use strict;
use Exporter ();
use NETWORK_REGION_MGW qw(load_network_region_config);

use AudLog;
use Data::Dumper;


#-------------------------------------------------------------------------------


#-------------------------------------------------------------------------------
# Modify/Add rules here
#-------------------------------------------------------------------------------
my %MONTH = (
    "01" => "Jan",
    "02" => "Feb",
    "03" => "Mar",
    "04" => "Apr",
    "05" => "May",
    "06" => "Jun",
    "07" => "Jul",
    "08" => "Aug",
    "09" => "Sep",
    "10" => "Oct",
    "11" => "Nov",
    "12" => "Dec"
);

#-------------------------------------------------------------------------------
# Replace special characters from string and strip leading and trailing blanks
#-------------------------------------------------------------------------------
sub strip_string {
    my ($in_string) = @_;
    $in_string =~ tr/\.\-\ \=\(\)/______/;
    $in_string =~ s/_+/_/g;
    $in_string =~ s/^_//;
    $in_string =~ s/_$//;
    return $in_string;
}

my $network_region_config = &NETWORK_REGION_MGW::load_network_region_config();

my @rules = 
(
    {
        RULE_TYPE => 'GENERIC_CSV_GATEWAY',
        RULE_DESC => 'Huawei MGW Interface',
        INPUT_FILE_DESCRIPTION => [ '^pmresult_10066.*' ],
        INPUT_DIR_DEPTH => 3,
        ORDER_OF_FILES => 'OLDEST_FIRST',
        FILENAME_HEADER_FIELDS     => {
            BLK_NAME => '^pmresult_(\d+)_\d+_\d+_\d+.csv$', ## get the block name from the second part of the raw file name
	},
        FIELD_SEPARATOR            => ',',
        HEADERLINES_TO_SKIP        => 0,
        HEADER_FIELDS              => [],
        DATA_FIELDS                => [0 .. 99],
        INVALID_VALUE_MATCH        => ['\r','\n', "'"], ## added special characters
        WHITESPACE_MODIFIER        => '_',
        FIELDS_TO_KEY_PIF_FILENAME => [],
        OUTPUT_BLOCK_NAME          => 'MGW',
        DEFAULT_NULL_VALUE         => '',
        UPPERCASE_COUNTERS         => 'True',
        
        HEADER_DATA_RECORD_PROCESSING => sub {
	         
            my ($blk_name_ref, $h_ref, $d_ref) = @_;
          
            ## dont process this line if RESULT_TIME is not available, 
            ## this will ignored line 2 of raw file which do not contains value for RESULT_TIME
	    return 1 if $d_ref->{'RESULT_TIME'} eq '';
	    ## dont process this line if OBJECT_NAME is blank
            if (exists($d_ref->{'OBJECT_NAME'})){
                return 1 if $d_ref->{'OBJECT_NAME'} eq '';
            }
            
            ## prefixed block name with 'M' 
            if ($h_ref->{'BLK_NAME'} != /^M\d+$/) {
                $h_ref->{'BLK_NAME'} = 'M'.$h_ref->{'BLK_NAME'};
            }
        
            $$blk_name_ref = $h_ref->{'BLK_NAME'};

            ## get the STARTTIME, ENDTIME and DAY
            if ( exists($d_ref->{'RESULT_TIME'}) ) {
                my ($startdate, $starttime) = split(/\_/, $d_ref->{'RESULT_TIME'});
                my ($year, $month, $day) = split(/\-/, $startdate);
                $startdate = $day.$MONTH{$month}.$year;
                $d_ref->{'DAY'} = $startdate;
                $d_ref->{'STARTTIME'} = $starttime;
            } 
 
            ##  replaced GRANULARITY_PERIOD with GP 
            if ( exists($d_ref->{'GRANULARITY_PERIOD'}) ) {
                $d_ref->{'GP'} = $d_ref->{'GRANULARITY_PERIOD'} * 60;  ## convert to seconds
                delete $d_ref->{'GRANULARITY_PERIOD'};
            }

            ## change RELIABILITY to 0 if the value is Reliable else change to 1
            if ($d_ref->{'RELIABILITY'} !~ /^\d+$/) {
                $d_ref->{'RELIABILITY'} = ($d_ref->{'RELIABILITY'} eq 'Reliable') ? 0 : 1;
            }

            foreach my $key (keys %{$d_ref}) {
	        	        
	        my $obj_id;
	        if ($key eq 'OBJECT_NAME') {
                    ## rename OBJECT_NAME to OBJECT
                    my $obj = $d_ref->{$key};
                    $obj =~ s/,_/,/g;  ## replace ',_' with ',' 
                    $d_ref->{'OBJECT'} = $obj;
                    delete $d_ref->{$key};
                     
                    ## split MGW_ID from the rest of the object segment
                    my @obj_name1 = split('/', $d_ref->{'OBJECT'});
                    $d_ref->{'MGW_ID'} = shift @obj_name1;

                    foreach my $object_block (@obj_name1) {
 
                        ## extract addtional information from OBJECT
                        my @object_segment = split(/:/, $object_block);
                        $obj_id = shift @object_segment;  ## get the object id

                        ## M3UARoute, MTP3Route, Local_IP, Remote_IP  do not have sub field, itself is the sub field
                        if ( ($obj_id =~ /^M3UARoute$/) || ($obj_id =~ /^MTP3Route$/) || 
                             ($obj_id =~ /^Local_IP$/) || ($obj_id =~ /^Remote_IP$/) ) {
                            $d_ref->{$obj_id} = $object_segment[0];
                        }
                        elsif (($obj_id =~ /^Frame$/) && ($h_ref->{'BLK_NAME'} eq 'M100666002')) {
                            ## Do nothing with Frame Object in Block M100666002 
                        }
                        ## all other object which is not MGWFunction
                        elsif (!($obj_id =~ /^MGWFunction$/)) { 
                            ## counters are seperated by ','
                            my @s_cpairs = split(/,/, $object_segment[0]);
                            foreach my $cpair (@s_cpairs) {
                                ## Counter name and value are separated by '=' 
                                my ($cname, $cvalue) = split(/=/, $cpair);
                                $cname = &EngineConfig::strip_string($cname);
                                $d_ref->{$cname} = $cvalue;
                            } 
                        }
                    }
                }
             
                if ($key =~ /^\d+$/) {
                    ## Raw files arriving in the form of counter ID, will be parsed as it is, prefix with 'MG'
                    my $tmp_key = "MG$key";
                    $d_ref->{$tmp_key} = $d_ref->{$key};
 	
                    delete $d_ref->{$key};
                }
	        
                if (exists $d_ref->{'MGW_ID'}) {
                    if (uc($d_ref->{'MGW_ID'}) eq $network_region_config->{uc($d_ref->{'MGW_ID'})}->{"MGW"})	{
                        ## if raw MGW_ID match the mapping file MGW, then create the respective NETWORK_ID & REGION_ID counter.
                        defined $d_ref->{'NETWORK_ID'} if ! exists $d_ref->{'NETWORK_ID'};
                        $d_ref->{'NETWORK_ID'} = $network_region_config->{uc($d_ref->{'MGW_ID'})}->{"NETWORK"};
					
                        defined $d_ref->{'REGION_ID'} if ! exists $d_ref->{'REGION_ID'};
                        $d_ref->{'REGION_ID'} = $network_region_config->{uc($d_ref->{'MGW_ID'})}->{"REGION"};
                    }
                    else {
                        defined $d_ref->{'NETWORK_ID'} if ! exists $d_ref->{'NETWORK_ID'};
                        $d_ref->{'NETWORK_ID'} = "UNDEFINED";
					
                        defined $d_ref->{'REGION_ID'} if ! exists $d_ref->{'REGION_ID'};
                        $d_ref->{'REGION_ID'} = "UNDEFINED";
                    }
                }      
            } ## end of foreach key
					 
            return 0;
             
        }, ## end of HEADER_DATA_RECORD_PROCESSING
    }, ## end of rule
);  


sub engine_config {
    return \@rules;
}

1;
